# KotonohaLink Plug-in
# Copyright 2005 CHEEBOW (Motokazu Sekine) All Right Reserved.
# $Id: KotonohaLink.pl,v 1.13 2005/06/01 09:22:45 cheebow Exp $

package MT::plugins::KotonohaLink;

use strict;

use MT::Template::Context;
use MT::ConfigMgr;

use Jcode;

############################################################################
my $WORD_FILE = './plugins/kotonohalink/KotonohaLink.dat';
############################################################################

my $PLUGIN_NAME = 'KotonohaLink';
my $VERSION = 'v0.50';
 
###----------------------------------------------------------------------###
if (MT->can('add_plugin')) {
    require MT::Plugin;
    my $plugin = new MT::Plugin( {
                        name => $PLUGIN_NAME . ' ' . $VERSION,
                        description => 'Plug-in which changes a character sequence with a link.',
                        dir_name => 'kotonohalink',
                        doc_link => 'http://cheebow.sub.jp/docmt/kotonohalink/kotonohalink.html',
                                } );
    MT->add_plugin($plugin);
}

###----------------------------------------------------------------------###
MT::Template::Context->add_global_filter(KotonohaLink => \&KotonohaLink );

###----------------------------------------------------------------------###
sub KotonohaLink {
    my ($text, $arg, $ctx) = @_;
    $arg or return $text;
    
    # PublishCharset
    my $cfg = MT::ConfigMgr->instance;
    my $charset = {'Shift_JIS'=>'sjis','ISO-2022-JP'=>'jis','EUC-JP'=> 'euc',
                   'UTF-8'=>'utf8'}->{$cfg->PublishCharset} || 'utf8';

    # Load Data
    our %wordList;
    open(DAT_FILE, $WORD_FILE) or return $text;
    while (<DAT_FILE>) {
        s/[\r?\n]*$//;
        my $line = Jcode->new($_)->$charset();
        my($word, $url) = split(/\t/, $line);
        $wordList{$word} = $url;
    }
    close(DAT_FILE);

    # Filter
    use HTML::Parser;
    
    my $p = HTML::Parser->new(
        api_version => 3,
        default_h => [\&default, "text"],
        start_h   => [\&start,   "tagname, text"],
        end_h     => [\&end,     "tagname, text"],
        text_h    => [\&text,    "text"],
        marked_sections => 1,
    );
    $p->unbroken_text(1);

    our $out_text = "";
    our $tmp_text = $text;

    foreach our $cur_word(keys(%wordList)) {
        our $add_link = 1;
        $out_text = "";
        $p->parse($tmp_text);
        $tmp_text = $out_text;
    }
    
    $out_text;
}

###----------------------------------------------------------------------###
sub default(){
    our $out_text .= shift;
}

sub start(){
    my ($tagname, $str) = @_;
    our ($add_link, $out_text);
    if($tagname eq "a"){
        $add_link = 0;
    }
    $out_text .= $str;
}

sub end(){
    my ($tagname, $str) = @_;
    our ($add_link, $out_text);
    if($tagname eq "a"){
        $add_link = 1;
    }
    $out_text .= $str;
}

sub text(){
    my $str = shift;
    our ($add_link, $out_text);
    &add_link(\$str) if $add_link;
    $out_text .= $str;
}

sub add_link(){
    my $str_ref = shift;
    our %wordList;
    our $cur_word;
    $$str_ref =~ s!($cur_word)!<a class="KotonohaLink" href="$wordList{$cur_word}">$1</a>!g;
}
